unit TntToolBarControl;

interface
uses
  Classes, ComCtrls, Controls, Messages, DsgnIntf,
  TntControls, TntClasses;

{
  Gabriel Corneanu:

  This is a unit to replace a standard TToolButton to a Unicode capable button.
  It is based on Tnt controls and follows the same ideas.
  A problem is how to simply exchange the controls from design to these versions
  WITHOUT registering as a new component and replacing all buttons on all forms.

  One solution is to use the ReadState method of a form to assign an OnFindComponent
  event on the reader and there replace a class against the new one.
  This envolves a lot of work, and there is another solution: the same class name is
  redefined here and it will be used if this class is used by the compiler
  INSTEAD of the original one;
  to achieve this, this unit must be used in the "uses" clause AFTER the original
  unit which defines the class (in this case "ComCtrls"), in the interface part too.

  The tool button has an extra property ("WholeDropDown") that enables the
  BTNS_WHOLEDROPDOWN style; read MSDN for details
}
{
  17/10/2010: Modified by Phung Tien Duong
  Its original name was UniTntCtrls
  Now you can install the component on Tnt Win32 page
  Drop the component on your form, right button click to create new button or separator.
  And you can set some propeties of toolbutton (like Caption, Hint..) at design time
}

type
  TTntToolButton = class(TToolButton)
  private
    FWholeDropDown: boolean;
    function  GetHint: WideString;
    procedure SetHint(const Value: WideString);
    procedure SetCaption(const Value: TWideCaption);
    function  GetCaption: TWideCaption;
    procedure SetWholeDropDown(const Value: boolean);
    function  IsCaptionStored: Boolean;
    function  IsHintStored: Boolean;
  protected
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure DefineProperties(Filer: TFiler); override;
    function  GetActionLinkClass: TControlActionLinkClass; override;
    procedure ActionChange(Sender: TObject; CheckDefaults: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property  Caption: TWideCaption read GetCaption write SetCaption stored IsCaptionStored;
    property  Hint: WideString read GetHint write SetHint stored IsHintStored;
    property  WholeDropDown: boolean read FWholeDropDown write SetWholeDropDown default false;
  end;

  TTntToolBar = class(TToolBar)
  private
    FTempWideString: WideString;
    function  CheckBtnMsg(var Message: TMessage): boolean;
    function  GetHint: WideString;
    function  IsHintStored: Boolean;
    procedure SetHint(const Value: WideString);
  protected
    procedure WndProc(var Message: TMessage); override;
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure DefineProperties(Filer: TFiler); override;
  public
  published
    property Hint: WideString read GetHint write SetHint stored IsHintStored;
  end;

  TTntToolbarEditor = class(TDefaultEditor)
  private
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string{TNT-ALLOW string}; override;
    function GetVerbCount: Integer; override;
  end;

procedure Register;

implementation

uses
  CommCtrl, TntActnList, TntSysUtils, SysUtils;

procedure Register;
begin
  RegisterComponents('Tnt Win32', [TTntToolBar]);
  RegisterClass(TTntToolButton);
  RegisterComponentEditor(TTntToolBar, TTntToolbarEditor);
end;

const
  BTNS_WHOLEDROPDOWN = $80;

resourcestring
  SNewButton = 'New Button';
  SNewSeparator = 'New Separator';

{ Methods of TTntToolButton }
constructor TTntToolButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FWholeDropDown := false;
end;

procedure TTntToolButton.ActionChange(Sender: TObject; CheckDefaults: Boolean);
begin
  inherited ActionChange(Sender, CheckDefaults);
  if Sender is TTntAction then
    with TTntAction(Sender) do
    begin
      if not CheckDefaults or (Self.Caption = '') or (Self.Caption = Self.Name) then
        Self.Caption := Caption;
      if not CheckDefaults or (Self.Hint = '') then
        Self.Hint := Hint;
    end;
end;

procedure TTntToolButton.CMHintShow(var Message: TMessage);
begin
  ProcessCMHintShowMsg(Message);
  inherited;
end;

function TTntToolButton.GetActionLinkClass: TControlActionLinkClass;
begin
  Result := TntControl_GetActionLinkClass(Self, TToolButtonActionLink);
end;

procedure TTntToolButton.SetCaption(const Value: TWideCaption);
begin
  TntControl_SetText(Self, Value);
end;

function TTntToolButton.GetHint: WideString;
begin
  Result := TntControl_GetHint(Self)
end;

function TTntToolButton.GetCaption: TWideCaption;
begin
  Result := TntControl_GetText(Self);
end;

procedure TTntToolButton.SetHint(const Value: WideString);
begin
  TntControl_SetHint(Self, Value);
end;

procedure TTntToolButton.SetWholeDropDown(const Value: boolean);
begin
  if FWholeDropDown <> Value then
  begin
    FWholeDropDown := Value;
    RefreshControl;
  end;
end;

function TTntToolButton.IsCaptionStored: Boolean;
begin
  Result := TntControl_IsCaptionStored(Self)
end;

function TTntToolButton.IsHintStored: Boolean;
begin
  Result := TntControl_IsHintStored(Self)
end;

procedure TTntToolButton.DefineProperties(Filer: TFiler);
begin
  inherited;
  TntPersistent_AfterInherited_DefineProperties(Filer, Self);
end;

{ TTntToolBar }

function TTntToolBar.CheckBtnMsg(var Message: TMessage): boolean;
var
  B : TTntToolButton;
  function CheckButton(ACtrl: TControl; var Style: Byte): boolean;
  begin
    Result := ACtrl.InheritsFrom(TTntToolButton);
    if Result then
    begin
      B := TTntToolButton(ACtrl);
      Result := B.FWholeDropDown;
      if Result and (GetComCtlVersion >= ComCtlVersionIE5) then
        Style := Style or BTNS_WHOLEDROPDOWN;
    end;
  end;
var
  BT: PTBButton;
  BI: PTBButtonInfoW;
  T : string;
begin
  Result := false;
  B := nil;
  if Message.Msg = TB_INSERTBUTTON then
  begin
    BT := PTBButton(Message.LParam);
    CheckButton(TControl(BT.dwData), BT.fsStyle);
    if (B <> nil) and ShowCaptions and (BT.iString <> -1) and Win32PlatformIsUnicode then
    begin
      FTempWideString := B.Caption;
      if (WideString(AnsiString(FTempWideString)) <> FTempWideString) then
      begin
        //here 2 nulls are needed
        FTempWideString := FTempWideString + #0;
        BT.iString := Self.Perform(TB_ADDSTRINGW, 0, Longint(PWideChar(FTempWideString)));
      end;
    end;
  end
  else if Message.Msg = TB_SETBUTTONINFO then
  begin
    BI := PTBButtonInfoW(Message.LParam);
    CheckButton(TControl(BI.lParam), BI.fsStyle);
    if (B <> nil) and ShowCaptions and (BI.cchText > 0) and Win32PlatformIsUnicode then
    begin
      T := PChar(BI.pszText);
      FTempWideString := B.Caption;
      if WideString(T) <> FTempWideString then
      begin
        Message.Msg := TB_SETBUTTONINFOW;
        BI.cchText  := Length(FTempWideString);
        BI.pszText  := PWideChar(FTempWideString);
      end;
    end;
  end;
end;

procedure TTntToolBar.CMHintShow(var Message: TMessage);
begin
  ProcessCMHintShowMsg(Message);
  inherited;
end;

procedure TTntToolBar.DefineProperties(Filer: TFiler);
begin
  inherited;
  TntPersistent_AfterInherited_DefineProperties(Filer, Self);
end;

function TTntToolBar.GetHint: WideString;
begin
  Result := TntControl_GetHint(Self);
end;

function TTntToolBar.IsHintStored: Boolean;
begin
  Result := TntControl_IsHintStored(Self);
end;

procedure TTntToolBar.SetHint(const Value: WideString);
begin
  TntControl_SetHint(Self, Value);
end;

procedure TTntToolBar.WndProc(var Message: TMessage);
var
  Handled : boolean;
begin
  Handled := CheckBtnMsg(Message);
  if not Handled then
    inherited WndProc(Message);
end;

{ TTntToolbarEditor }

procedure TTntToolbarEditor.ExecuteVerb(Index: Integer);
  procedure CreateNewToolButton(style: TToolButtonStyle);
  var
    button: TTntToolButton;
    toolbar: TTntToolBar;
  begin
    toolbar := TTntToolBar(Component);
    button := TTntToolButton.Create(toolbar.Owner);
    button.Parent := toolbar;
    button.Name := Designer.UniqueName('TTntToolButton');
    button.Left := toolbar.Buttons[toolbar.ButtonCount - 1].Left + 1;
    button.Caption := button.Name;
    button.Style := style;
    if style = tbsSeparator then button.Width := 8
  end;
begin
  case Index of
    0: CreateNewToolButton(tbsButton);
    1: CreateNewToolButton(tbsSeparator);
  end;
end;

function TTntToolbarEditor.GetVerb(Index: Integer): string{TNT-ALLOW string};
begin
  case Index of
    0: Result := SNewButton;
    1: Result := SNewSeparator;
  end;
end;

function TTntToolbarEditor.GetVerbCount: Integer;
begin
  Result := 2;
end;

end.
